<?php

/**
 * @file
 * Admin page callback file for the languageassign module.
 */


/**
 * Users
 */

require_once drupal_get_path('module', 'user') . '/user.admin.inc';

/**
 * We want to add a Language column to the user table at admin/user/user.
 * Unfortunately, the list of columns in that table is hard-coded in TWO 
 * functions, user_admin_account() and theme_user_admin_account(), both in
 * user.admin.inc.  So override and entirely replicate both of those here,
 * with the Language column added.
 */

/**
 * Page callback for admin/user/user.
 * 
 * @param unknown_type $callback_arg
 * This function is identical to user_admin() in user.admin.inc,
 * except that it calls our custom form builder, languageassign_user_admin_account().
 */
function languageassign_user_admin($callback_arg = '') {
  $op = isset($_POST['op']) ? $_POST['op'] : $callback_arg;

  switch ($op) {
    case t('Create new account'):
    case 'create':
      $output = drupal_get_form('user_register');
      break;
    default:
      if (!empty($_POST['accounts']) && isset($_POST['operation']) && ($_POST['operation'] == 'delete')) {
        $output = drupal_get_form('user_multiple_delete_confirm');
      }
      else {
        $output = drupal_get_form('user_filter_form');
        $output .= drupal_get_form('languageassign_user_admin_account'); // <-- N.B.
      }
  }
  return $output;
}

/**
 * Form builder; User administration page.
 *
 * @ingroup forms
 * @see user_admin_account()
 * @see languageassign_user_admin_account_validate()
 * @see languageassign_user_admin_account_submit()
 *
 * This function is identical to user_admin_account() in user.admin.inc,
 * except that we:
 * - Add a (sortable) Language column to the user table
 * - Separate out the language assignment operations, relabel them,
 *   and group them under a separate "Set language" optgroup.
 */
function languageassign_user_admin_account() {
  $filter = user_build_filter_query();

  $header = array(
    array(),
    array('data' => t('Username'), 'field' => 'u.name'),
    array('data' => t('Status'), 'field' => 'u.status'),
    t('Roles'),
    array('data' => t('Member for'), 'field' => 'u.created', 'sort' => 'desc'),
    array('data' => t('Last access'), 'field' => 'u.access'),
    array('data' => t('Language'), 'field' => 'u.language'),
    t('Operations')
  );

  if ($filter['join'] != "") {
    $sql = 'SELECT DISTINCT u.uid, u.name, u.status, u.created, u.access, u.language FROM {users} u LEFT JOIN {users_roles} ur ON u.uid = ur.uid '. $filter['join'] .' WHERE u.uid <> 0 '. $filter['where'];
    $query_count = 'SELECT COUNT(DISTINCT u.uid) FROM {users} u LEFT JOIN {users_roles} ur ON u.uid = ur.uid '. $filter['join'] .' WHERE u.uid <> 0 '. $filter['where'];
  }
  else {
    $sql = 'SELECT u.uid, u.name, u.status, u.created, u.access, u.language FROM {users} u WHERE u.uid <> 0 '. $filter['where'];
    $query_count = 'SELECT COUNT(u.uid) FROM {users} u WHERE u.uid <> 0 '. $filter['where'];
  }

  $sql .= tablesort_sql($header);

  $result = pager_query($sql, 50, 0, $query_count, $filter['args']);

  $form['options'] = array(
    '#type' => 'fieldset',
    '#title' => t('Update options'),
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
  );
  // Retrieve user operations; separate out language assignment operations
  // and put them into a separate optgroup.
  $options = array();
  $la_opts = array();
  foreach (module_invoke_all('user_operations') as $operation => $array) {
    if (substr($operation, 0, 15) == 'languageassign-') {
      $langcode = substr($operation, 15);
      $la_opts[$operation] = locale_language_name($langcode);
    }
    else {
      $options[$operation] = $array['label'];
    }
  }
  $options += array(t('Set language') => $la_opts);
  $form['options']['operation'] = array(
    '#type' => 'select',
    '#options' => $options,
    '#default_value' => 'unblock',
  );
  $form['options']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Update'),
  );

  $destination = drupal_get_destination();

  $status = array(t('blocked'), t('active'));
  $roles = user_roles(TRUE);
  $accounts = array();
  while ($account = db_fetch_object($result)) {
    $accounts[$account->uid] = '';
    $form['name'][$account->uid] = array('#value' => theme('username', $account));
    $form['status'][$account->uid] =  array('#value' => $status[$account->status]);
    $users_roles = array();
    $roles_result = db_query('SELECT rid FROM {users_roles} WHERE uid = %d', $account->uid);
    while ($user_role = db_fetch_object($roles_result)) {
      $users_roles[] = $roles[$user_role->rid];
    }
    asort($users_roles);
    $form['roles'][$account->uid][0] = array('#value' => theme('item_list', $users_roles));
    $form['member_for'][$account->uid] = array('#value' => format_interval(time() - $account->created));
    $form['last_access'][$account->uid] =  array('#value' => $account->access ? t('@time ago', array('@time' => format_interval(time() - $account->access))) : t('never'));
    $form['language'][$account->uid] = array('#value' => locale_language_name($account->language));
    $form['operations'][$account->uid] = array('#value' => l(t('edit'), "user/$account->uid/edit", array('query' => $destination)));
  }
  $form['accounts'] = array(
    '#type' => 'checkboxes',
    '#options' => $accounts
  );
  $form['pager'] = array('#value' => theme('pager', NULL, 50, 0));
  
  // Process hooks on user_admin_account().
  $form_state = array();
  drupal_prepare_form('user_admin_account', $form, $form_state);
  
  return $form;
}

/**
 * Form submission handler for languageassign_user_admin_account().
 *
 * @see languageassign_user_admin_account()
 * @see languageassign_user_admin_account_validate()
 */
function languageassign_user_admin_account_submit($form, &$form_state) {
  return user_admin_account_submit($form, $form_state);
}

/**
 * Form validation handler for languageassign_user_admin_account().
 *
 * @see languageassign_user_admin_account()
 * @see languageassign_user_admin_account_submit()
 */
function languageassign_user_admin_account_validate($form, &$form_state) {
  return user_admin_account_validate($form, $form_state);
}

/**
 * Theme user administration overview.
 *
 * @ingroup themeable
 * This function is identical to theme_user_admin_account() in user.module, 
 * except that we add a Language column to the user table.
 */
function theme_languageassign_user_admin_account($form) {
  // Overview table:
  $header = array(
    theme('table_select_header_cell'),
    array('data' => t('Username'), 'field' => 'u.name'),
    array('data' => t('Status'), 'field' => 'u.status'),
    t('Roles'),
    array('data' => t('Member for'), 'field' => 'u.created', 'sort' => 'desc'),
    array('data' => t('Last access'), 'field' => 'u.access'),
    array('data' => t('Language'), 'field' => 'u.language'),
    t('Operations')
  );

  $output = drupal_render($form['options']);
  if (isset($form['name']) && is_array($form['name'])) {
    foreach (element_children($form['name']) as $key) {
      $rows[] = array(
        drupal_render($form['accounts'][$key]),
        drupal_render($form['name'][$key]),
        drupal_render($form['status'][$key]),
        drupal_render($form['roles'][$key]),
        drupal_render($form['member_for'][$key]),
        drupal_render($form['last_access'][$key]),
        drupal_render($form['language'][$key]),
        drupal_render($form['operations'][$key]),
      );
    }
  }
  else {
    $rows[] = array(array('data' => t('No users available.'), 'colspan' => '7'));
  }

  $output .= theme('table', $header, $rows);
  if ($form['pager']['#value']) {
    $output .= drupal_render($form['pager']);
  }

  $output .= drupal_render($form);

  return $output;
}
